@@ -146,7 +146,7 @@ def substitute_config_variable(template_str, config_variables_dict):
     if match:
         config_variable_value = config_variables_dict.get(match.group(1))
 
-        if config_variable_value:
+        if config_variable_value is not None:
             if match.start() == 0 and match.end() == len(template_str):
                 return config_variable_value
             else:
