@@ -1,14 +1,15 @@
-import logging
 import datetime
+import logging
 from string import Template
+from urllib.parse import urlparse
 
-from great_expectations.datasource import Datasource
-from great_expectations.datasource.types import BatchMarkers
-from great_expectations.dataset.sqlalchemy_dataset import SqlAlchemyBatchReference
-from great_expectations.exceptions import DatasourceInitializationError
-from great_expectations.types import ClassConfig
 from great_expectations.core.batch import Batch
 from great_expectations.core.util import nested_update
+from great_expectations.dataset.sqlalchemy_dataset import SqlAlchemyBatchReference
+from great_expectations.datasource import Datasource
+from great_expectations.datasource.types import BatchMarkers
+from great_expectations.exceptions import DatasourceInitializationError
+from great_expectations.types import ClassConfig
 from great_expectations.types.configurations import classConfigSchema
 
 logger = logging.getLogger(__name__)
@@ -22,6 +23,24 @@ except ImportError:
     logger.debug("Unable to import sqlalchemy.")
 
 
+if sqlalchemy != None:
+    try:
+        import google.auth
+
+        datasource_initialization_exceptions = (
+            sqlalchemy.exc.OperationalError,
+            sqlalchemy.exc.DatabaseError,
+            sqlalchemy.exc.ArgumentError,
+            google.auth.exceptions.GoogleAuthError,
+        )
+    except ImportError:
+        datasource_initialization_exceptions = (
+            sqlalchemy.exc.OperationalError,
+            sqlalchemy.exc.DatabaseError,
+            sqlalchemy.exc.ArgumentError,
+        )
+
+
 class SqlAlchemyDatasource(Datasource):
     """
     A SqlAlchemyDatasource will provide data_assets converting batch_kwargs using the following rules:
@@ -31,10 +50,13 @@ class SqlAlchemyDatasource(Datasource):
         that query. The query can be parameterized according to the standard python Template engine, which
         uses $parameter, with additional kwargs passed to the get_batch method.
     """
-    recognized_batch_parameters = {'query_parameters', 'limit', 'dataset_options'}
+
+    recognized_batch_parameters = {"query_parameters", "limit", "dataset_options"}
 
     @classmethod
-    def build_configuration(cls, data_asset_type=None, batch_kwargs_generators=None, **kwargs):
+    def build_configuration(
+        cls, data_asset_type=None, batch_kwargs_generators=None, **kwargs
+    ):
         """
         Build a full configuration object for a datasource, potentially including generators with defaults.
 
@@ -51,7 +73,7 @@ class SqlAlchemyDatasource(Datasource):
         if data_asset_type is None:
             data_asset_type = {
                 "class_name": "SqlAlchemyDataset",
-                "module_name": "great_expectations.dataset"
+                "module_name": "great_expectations.dataset",
             }
         else:
             data_asset_type = classConfigSchema.dump(ClassConfig(**data_asset_type))
@@ -63,24 +85,37 @@ class SqlAlchemyDatasource(Datasource):
 
         return configuration
 
-    def __init__(self, name="default", data_context=None, data_asset_type=None, credentials=None, batch_kwargs_generators=None, **kwargs):
+    def __init__(
+        self,
+        name="default",
+        data_context=None,
+        data_asset_type=None,
+        credentials=None,
+        batch_kwargs_generators=None,
+        **kwargs
+    ):
         if not sqlalchemy:
-            raise DatasourceInitializationError(name, "ModuleNotFoundError: No module named 'sqlalchemy'")
+            raise DatasourceInitializationError(
+                name, "ModuleNotFoundError: No module named 'sqlalchemy'"
+            )
 
-        configuration_with_defaults = SqlAlchemyDatasource.build_configuration(data_asset_type, batch_kwargs_generators, **kwargs)
+        configuration_with_defaults = SqlAlchemyDatasource.build_configuration(
+            data_asset_type, batch_kwargs_generators, **kwargs
+        )
         data_asset_type = configuration_with_defaults.pop("data_asset_type")
-        batch_kwargs_generators = configuration_with_defaults.pop("batch_kwargs_generators", None)
+        batch_kwargs_generators = configuration_with_defaults.pop(
+            "batch_kwargs_generators", None
+        )
         super(SqlAlchemyDatasource, self).__init__(
             name,
             data_context=data_context,
             data_asset_type=data_asset_type,
             batch_kwargs_generators=batch_kwargs_generators,
-            **configuration_with_defaults)
+            **configuration_with_defaults
+        )
 
         if credentials is not None:
-            self._datasource_config.update({
-                "credentials": credentials
-            })
+            self._datasource_config.update({"credentials": credentials})
         else:
             credentials = {}
 
@@ -96,9 +131,7 @@ class SqlAlchemyDatasource(Datasource):
                 self.engine.connect()
             elif "url" in credentials:
                 url = credentials.pop("url")
-                # TODO perhaps we could carefully regex out the driver from the
-                #  url. It would need to be cautious to avoid leaking secrets.
-                self.drivername = "other"
+                self.drivername = urlparse(url).scheme
                 self.engine = create_engine(url, **kwargs)
                 self.engine.connect()
 
@@ -109,7 +142,7 @@ class SqlAlchemyDatasource(Datasource):
                 self.engine = create_engine(options)
                 self.engine.connect()
 
-        except (sqlalchemy.exc.OperationalError, sqlalchemy.exc.DatabaseError) as sqlalchemy_error:
+        except datasource_initialization_exceptions as sqlalchemy_error:
             raise DatasourceInitializationError(self._name, str(sqlalchemy_error))
 
         self._build_generators()
@@ -131,16 +164,22 @@ class SqlAlchemyDatasource(Datasource):
             drivername = credentials.pop("drivername")
             schema_name = credentials.pop("schema_name", None)
             if schema_name is not None:
-                logger.warning("schema_name specified creating a URL with schema is not supported. Set a default "
-                               "schema on the user connecting to your database.")
+                logger.warning(
+                    "schema_name specified creating a URL with schema is not supported. Set a default "
+                    "schema on the user connecting to your database."
+                )
             options = sqlalchemy.engine.url.URL(drivername, **credentials)
         return options, drivername
 
     def get_batch(self, batch_kwargs, batch_parameters=None):
         # We need to build a batch_id to be used in the dataframe
-        batch_markers = BatchMarkers({
-            "ge_load_time": datetime.datetime.utcnow().strftime("%Y%m%dT%H%M%S.%fZ")
-        })
+        batch_markers = BatchMarkers(
+            {
+                "ge_load_time": datetime.datetime.now(datetime.timezone.utc).strftime(
+                    "%Y%m%dT%H%M%S.%fZ"
+                )
+            }
+        )
 
         if "bigquery_temp_table" in batch_kwargs:
             query_support_table_name = batch_kwargs.get("bigquery_temp_table")
@@ -152,38 +191,66 @@ class SqlAlchemyDatasource(Datasource):
 
         if "query" in batch_kwargs:
             if "limit" in batch_kwargs or "offset" in batch_kwargs:
-                logger.warning("Limit and offset parameters are ignored when using query-based batch_kwargs; consider "
-                               "adding limit and offset directly to the generated query.")
+                logger.warning(
+                    "Limit and offset parameters are ignored when using query-based batch_kwargs; consider "
+                    "adding limit and offset directly to the generated query."
+                )
             if "query_parameters" in batch_kwargs:
-                query = Template(batch_kwargs["query"]).safe_substitute(batch_kwargs["query_parameters"])
+                query = Template(batch_kwargs["query"]).safe_substitute(
+                    batch_kwargs["query_parameters"]
+                )
             else:
                 query = batch_kwargs["query"]
-            batch_reference = SqlAlchemyBatchReference(engine=self.engine, query=query, table_name=query_support_table_name,
-                                                       schema=batch_kwargs.get("schema"))
+            batch_reference = SqlAlchemyBatchReference(
+                engine=self.engine,
+                query=query,
+                table_name=query_support_table_name,
+                schema=batch_kwargs.get("schema"),
+            )
         elif "table" in batch_kwargs:
-            table = batch_kwargs['table']
-            limit = batch_kwargs.get('limit')
-            offset = batch_kwargs.get('offset')
+            table = batch_kwargs["table"]
+            limit = batch_kwargs.get("limit")
+            offset = batch_kwargs.get("offset")
             if limit is not None or offset is not None:
-                logger.info("Generating query from table batch_kwargs based on limit and offset")
+                logger.info(
+                    "Generating query from table batch_kwargs based on limit and offset"
+                )
                 # In BigQuery the table name is already qualified with its schema name
                 if self.engine.dialect.name.lower() == "bigquery":
                     schema = None
                 else:
                     schema = batch_kwargs.get("schema")
-                raw_query = sqlalchemy.select([sqlalchemy.text("*")])\
-                    .select_from(sqlalchemy.schema.Table(table, sqlalchemy.MetaData(),
-                                                         schema=schema))\
-                    .offset(offset)\
+                raw_query = (
+                    sqlalchemy.select([sqlalchemy.text("*")])
+                    .select_from(
+                        sqlalchemy.schema.Table(
+                            table, sqlalchemy.MetaData(), schema=schema
+                        )
+                    )
+                    .offset(offset)
                     .limit(limit)
-                query = str(raw_query.compile(self.engine, compile_kwargs={"literal_binds": True}))
-                batch_reference = SqlAlchemyBatchReference(engine=self.engine, query=query, table_name=query_support_table_name,
-                                                           schema=batch_kwargs.get("schema"))
+                )
+                query = str(
+                    raw_query.compile(
+                        self.engine, compile_kwargs={"literal_binds": True}
+                    )
+                )
+                batch_reference = SqlAlchemyBatchReference(
+                    engine=self.engine,
+                    query=query,
+                    table_name=query_support_table_name,
+                    schema=batch_kwargs.get("schema"),
+                )
             else:
-                batch_reference = SqlAlchemyBatchReference(engine=self.engine, table_name=table,
-                                                           schema=batch_kwargs.get("schema"))
+                batch_reference = SqlAlchemyBatchReference(
+                    engine=self.engine,
+                    table_name=table,
+                    schema=batch_kwargs.get("schema"),
+                )
         else:
-            raise ValueError("Invalid batch_kwargs: exactly one of 'table' or 'query' must be specified")
+            raise ValueError(
+                "Invalid batch_kwargs: exactly one of 'table' or 'query' must be specified"
+            )
 
         return Batch(
             datasource_name=self.name,
@@ -191,13 +258,14 @@ class SqlAlchemyDatasource(Datasource):
             data=batch_reference,
             batch_parameters=batch_parameters,
             batch_markers=batch_markers,
-            data_context=self._data_context
+            data_context=self._data_context,
         )
 
-    def process_batch_parameters(self, query_parameters=None, limit=None, dataset_options=None):
+    def process_batch_parameters(
+        self, query_parameters=None, limit=None, dataset_options=None
+    ):
         batch_kwargs = super(SqlAlchemyDatasource, self).process_batch_parameters(
-            limit=limit,
-            dataset_options=dataset_options,
+            limit=limit, dataset_options=dataset_options,
         )
         nested_update(batch_kwargs, {"query_parameters": query_parameters})
         return batch_kwargs
