@@ -1,33 +1,82 @@
-import datetime
 import logging
+import warnings
 from collections import OrderedDict
 
-from great_expectations.data_asset import DataAsset
-from great_expectations.data_context.util import instantiate_class_from_config
-from great_expectations.exceptions import ClassInstantiationError
+from dateutil.parser import ParserError, parse
 
-from ..data_context.types.resource_identifiers import (
+from great_expectations.core import RunIdentifier
+from great_expectations.data_asset import DataAsset
+from great_expectations.data_context.types.resource_identifiers import (
     ExpectationSuiteIdentifier,
     ValidationResultIdentifier,
 )
+from great_expectations.data_context.util import instantiate_class_from_config
+from great_expectations.exceptions import ClassInstantiationError
+from great_expectations.validation_operators.types.validation_operator_result import (
+    ValidationOperatorResult,
+)
+
 from .util import send_slack_notification
 
 logger = logging.getLogger(__name__)
 
 
-# NOTE: Abe 2019/08/24 : This is first implementation of all these classes. Consider them UNSTABLE for now.
+logger = logging.getLogger(__name__)
 
 
 class ValidationOperator(object):
     """
     The base class of all validation operators.
 
-    It defines the signature of the public run method - this is the only
-    contract re operators' API. Everything else is up to the implementors
+    It defines the signature of the public run method. This method and the validation_operator_config property are the
+    only contract re operators' API. Everything else is up to the implementors
     of validation operator classes that will be the descendants of this base class.
     """
 
-    def run(self, assets_to_validate, run_id, evaluation_parameters=None):
+    def __init__(self) -> None:
+        self._validation_operator_config = None
+
+    @property
+    def validation_operator_config(self):
+        """
+        This method builds the config dict of a particular validation operator. The "kwargs" key is what really
+        distinguishes different validation operators.
+
+        e.g.:
+        {
+            "class_name": "ActionListValidationOperator",
+            "module_name": "great_expectations.validation_operators",
+            "name": self.name,
+            "kwargs": {
+                "action_list": self.action_list
+            },
+        }
+
+        {
+            "class_name": "WarningAndFailureExpectationSuitesValidationOperator",
+            "module_name": "great_expectations.validation_operators",
+            "name": self.name,
+            "kwargs": {
+                "action_list": self.action_list,
+                "base_expectation_suite_name": self.base_expectation_suite_name,
+                "expectation_suite_name_suffixes": self.expectation_suite_name_suffixes,
+                "stop_on_first_error": self.stop_on_first_error,
+                "slack_webhook": self.slack_webhook,
+                "notify_on": self.notify_on,
+            },
+        }
+        """
+
+        raise NotImplementedError
+
+    def run(
+        self,
+        assets_to_validate,
+        run_id=None,
+        evaluation_parameters=None,
+        run_name=None,
+        run_time=None,
+    ):
         raise NotImplementedError
 
 
@@ -67,8 +116,10 @@ class ActionListValidationOperator(ValidationOperator):
                     class_name: SlackRenderer
     """
 
-    def __init__(self, data_context, action_list):
+    def __init__(self, data_context, action_list, name):
+        super().__init__()
         self.data_context = data_context
+        self.name = name
 
         self.action_list = action_list
         self.actions = OrderedDict()
@@ -97,6 +148,17 @@ class ActionListValidationOperator(ValidationOperator):
                 )
             self.actions[action_config["name"]] = new_action
 
+    @property
+    def validation_operator_config(self) -> dict:
+        if self._validation_operator_config is None:
+            self._validation_operator_config = {
+                "class_name": "ActionListValidationOperator",
+                "module_name": "great_expectations.validation_operators",
+                "name": self.name,
+                "kwargs": {"action_list": self.action_list},
+            }
+        return self._validation_operator_config
+
     def _build_batch_from_item(self, item):
         """Internal helper method to take an asset to validate, which can be either:
           (1) a DataAsset; or
@@ -125,28 +187,54 @@ class ActionListValidationOperator(ValidationOperator):
 
         return batch
 
-    def run(self, assets_to_validate, run_id, evaluation_parameters=None):
-        result_object = {"success": None, "details": {}}
+    def run(
+        self,
+        assets_to_validate,
+        run_id=None,
+        evaluation_parameters=None,
+        run_name=None,
+        run_time=None,
+    ):
+        assert not (run_id and run_name) and not (
+            run_id and run_time
+        ), "Please provide either a run_id or run_name and/or run_time."
+        if isinstance(run_id, str) and not run_name:
+            warnings.warn(
+                "String run_ids will be deprecated in the future. Please provide a run_id of type "
+                "RunIdentifier(run_name=None, run_time=None), or a dictionary containing run_name "
+                "and run_time (both optional). Instead of providing a run_id, you may also provide"
+                "run_name and run_time separately.",
+                DeprecationWarning,
+            )
+            try:
+                run_time = parse(run_id)
+            except (ParserError, TypeError):
+                pass
+            run_id = RunIdentifier(run_name=run_id, run_time=run_time)
+        elif isinstance(run_id, dict):
+            run_id = RunIdentifier(**run_id)
+        elif not isinstance(run_id, RunIdentifier):
+            run_id = RunIdentifier(run_name=run_name, run_time=run_time)
+
+        run_results = {}
 
         for item in assets_to_validate:
+            run_result_obj = {}
             batch = self._build_batch_from_item(item)
             expectation_suite_identifier = ExpectationSuiteIdentifier(
                 expectation_suite_name=batch._expectation_suite.expectation_suite_name
             )
-            # validation_result_id = ValidationResultIdentifier(
-            #     batch_identifier=BatchIdentifier(batch.batch_id),
-            #     expectation_suite_identifier=expectation_suite_identifier,
-            #     run_id=run_id,
-            # )
-            result_object["details"][expectation_suite_identifier] = {}
+            validation_result_id = ValidationResultIdentifier(
+                batch_identifier=batch.batch_id,
+                expectation_suite_identifier=expectation_suite_identifier,
+                run_id=run_id,
+            )
             batch_validation_result = batch.validate(
                 run_id=run_id,
                 result_format="SUMMARY",
                 evaluation_parameters=evaluation_parameters,
             )
-            result_object["details"][expectation_suite_identifier][
-                "validation_result"
-            ] = batch_validation_result
+            run_result_obj["validation_result"] = batch_validation_result
             batch_actions_results = self._run_actions(
                 batch,
                 expectation_suite_identifier,
@@ -154,19 +242,16 @@ class ActionListValidationOperator(ValidationOperator):
                 batch_validation_result,
                 run_id,
             )
-            result_object["details"][expectation_suite_identifier][
-                "actions_results"
-            ] = batch_actions_results
+            run_result_obj["actions_results"] = batch_actions_results
+            run_results[validation_result_id] = run_result_obj
 
-        result_object["success"] = all(
-            [
-                val["validation_result"].success
-                for val in result_object["details"].values()
-            ]
+        return ValidationOperatorResult(
+            run_id=run_id,
+            run_results=run_results,
+            validation_operator_config=self.validation_operator_config,
+            evaluation_parameters=evaluation_parameters,
         )
 
-        return result_object
-
     def _run_actions(
         self,
         batch,
@@ -217,34 +302,6 @@ class ActionListValidationOperator(ValidationOperator):
 
         return batch_actions_results
 
-        # TODO: Note that the following code is unreachable
-        result_object = {}
-
-        for item in assets_to_validate:
-            batch = self._build_batch_from_item(item)
-            expectation_suite_identifier = ExpectationSuiteIdentifier(
-                expectation_suite_name=batch.expectation_suite_name
-            )
-            validation_result_id = ValidationResultIdentifier(
-                expectation_suite_identifier=expectation_suite_identifier,
-                run_id=run_id,
-                batch_identifier=batch.batch_id,
-            )
-            result_object[validation_result_id] = {}
-            batch_validation_result = batch.validate(result_format="SUMMARY")
-            result_object[validation_result_id][
-                "validation_result"
-            ] = batch_validation_result
-            batch_actions_results = self._run_actions(
-                batch, batch._expectation_suite, batch_validation_result, run_id
-            )
-            result_object[validation_result_id][
-                "actions_results"
-            ] = batch_actions_results
-
-        # NOTE: Eugene: 2019-09-24: Need to define this result object. Discussion required!
-        return result_object
-
 
 class WarningAndFailureExpectationSuitesValidationOperator(
     ActionListValidationOperator
@@ -323,6 +380,7 @@ class WarningAndFailureExpectationSuitesValidationOperator(
         self,
         data_context,
         action_list,
+        name,
         base_expectation_suite_name=None,
         expectation_suite_name_suffixes=None,
         stop_on_first_error=False,
@@ -330,7 +388,7 @@ class WarningAndFailureExpectationSuitesValidationOperator(
         notify_on="all",
     ):
         super(WarningAndFailureExpectationSuitesValidationOperator, self).__init__(
-            data_context, action_list,
+            data_context, action_list, name
         )
 
         if expectation_suite_name_suffixes is None:
@@ -347,23 +405,47 @@ class WarningAndFailureExpectationSuitesValidationOperator(
         self.slack_webhook = slack_webhook
         self.notify_on = notify_on
 
-    def _build_slack_query(self, run_return_obj):
-        timestamp = datetime.datetime.strftime(datetime.datetime.now(), "%x %X")
-        success = run_return_obj.get("success")
-        status_text = "Success :tada:" if success else "Failed :x:"
-        run_id = run_return_obj.get("run_id")
-        batch_identifiers = run_return_obj.get("batch_identifiers")
-        failed_data_assets = []
+    @property
+    def validation_operator_config(self) -> dict:
+        if self._validation_operator_config is None:
+            self._validation_operator_config = {
+                "class_name": "WarningAndFailureExpectationSuitesValidationOperator",
+                "module_name": "great_expectations.validation_operators",
+                "name": self.name,
+                "kwargs": {
+                    "action_list": self.action_list,
+                    "base_expectation_suite_name": self.base_expectation_suite_name,
+                    "expectation_suite_name_suffixes": self.expectation_suite_name_suffixes,
+                    "stop_on_first_error": self.stop_on_first_error,
+                    "slack_webhook": self.slack_webhook,
+                    "notify_on": self.notify_on,
+                },
+            }
+        return self._validation_operator_config
 
-        if run_return_obj.get("failure"):
-            failed_data_assets = [
+    def _build_slack_query(self, validation_operator_result: ValidationOperatorResult):
+        success = validation_operator_result.success
+        status_text = "Success :tada:" if success else "Failed :x:"
+        run_id = validation_operator_result.run_id
+        run_name = run_id.run_name
+        run_time = run_id.run_time.strftime("%x %X")
+        batch_identifiers = sorted(validation_operator_result.list_batch_identifiers())
+        failed_data_assets_msg_strings = []
+
+        run_results = validation_operator_result.run_results
+        failure_level_run_results = {
+            validation_result_identifier: run_result
+            for validation_result_identifier, run_result in run_results.items()
+            if run_result["expectation_suite_severity_level"] == "failure"
+        }
+
+        if failure_level_run_results:
+            failed_data_assets_msg_strings = [
                 validation_result_identifier.expectation_suite_identifier.expectation_suite_name
                 + "-"
                 + validation_result_identifier.batch_identifier
-                for validation_result_identifier, value in run_return_obj.get(
-                    "failure"
-                ).items()
-                if not value["validation_result"].success
+                for validation_result_identifier, run_result in failure_level_run_results.items()
+                if not run_result["validation_result"].success
             ]
 
         title_block = {
@@ -397,22 +479,25 @@ class WarningAndFailureExpectationSuitesValidationOperator(
                 "type": "section",
                 "text": {
                     "type": "mrkdwn",
-                    "text": "*Failed Batches:* {}".format(failed_data_assets),
+                    "text": "*Failed Batches:* {}".format(
+                        failed_data_assets_msg_strings
+                    ),
                 },
             }
             query["blocks"].append(failed_data_assets_element)
 
-        run_id_element = {
+        run_name_element = {
             "type": "section",
-            "text": {"type": "mrkdwn", "text": "*Run ID:* {}".format(run_id),},
+            "text": {"type": "mrkdwn", "text": "*Run Name:* {}".format(run_name),},
         }
-        query["blocks"].append(run_id_element)
+        query["blocks"].append(run_name_element)
 
-        timestamp_element = {
+        run_time_element = {
             "type": "section",
-            "text": {"type": "mrkdwn", "text": "*Timestamp:* {}".format(timestamp),},
+            "text": {"type": "mrkdwn", "text": "*Run Time:* {}".format(run_time),},
         }
-        query["blocks"].append(timestamp_element)
+        query["blocks"].append(run_time_element)
+
         query["blocks"].append(divider_block)
 
         documentation_url = "https://docs.greatexpectations.io/en/latest/reference/validation_operators/warning_and_failure_expectation_suites_validation_operator.html"
@@ -434,10 +519,33 @@ class WarningAndFailureExpectationSuitesValidationOperator(
     def run(
         self,
         assets_to_validate,
-        run_id,
+        run_id=None,
         base_expectation_suite_name=None,
         evaluation_parameters=None,
+        run_name=None,
+        run_time=None,
     ):
+        assert not (run_id and run_name) and not (
+            run_id and run_time
+        ), "Please provide either a run_id or run_name and/or run_time."
+        if isinstance(run_id, str) and not run_name:
+            warnings.warn(
+                "String run_ids will be deprecated in the future. Please provide a run_id of type "
+                "RunIdentifier(run_name=None, run_time=None), or a dictionary containing run_name "
+                "and run_time (both optional). Instead of providing a run_id, you may also provide"
+                "run_name and run_time separately.",
+                DeprecationWarning,
+            )
+            try:
+                run_time = parse(run_id)
+            except (ParserError, TypeError):
+                pass
+            run_id = RunIdentifier(run_name=run_id, run_time=run_time)
+        elif isinstance(run_id, dict):
+            run_id = RunIdentifier(**run_id)
+        elif not isinstance(run_id, RunIdentifier):
+            run_id = RunIdentifier(run_name=run_name, run_time=run_time)
+
         if base_expectation_suite_name is None:
             if self.base_expectation_suite_name is None:
                 raise ValueError(
@@ -445,13 +553,7 @@ class WarningAndFailureExpectationSuitesValidationOperator(
                 )
             base_expectation_suite_name = self.base_expectation_suite_name
 
-        return_obj = {
-            "batch_identifiers": [],
-            "success": None,
-            "failure": {},
-            "warning": {},
-            "run_id": run_id,
-        }
+        run_results = {}
 
         for item in assets_to_validate:
             batch = self._build_batch_from_item(item)
@@ -462,8 +564,6 @@ class WarningAndFailureExpectationSuitesValidationOperator(
             assert not batch_id is None
             assert not run_id is None
 
-            return_obj["batch_identifiers"].append(batch_id)
-
             failure_expectation_suite_identifier = ExpectationSuiteIdentifier(
                 expectation_suite_name=base_expectation_suite_name
                 + self.expectation_suite_name_suffixes[0]
@@ -493,15 +593,13 @@ class WarningAndFailureExpectationSuitesValidationOperator(
                 )
 
             if failure_expectation_suite:
-                return_obj["failure"][failure_validation_result_id] = {}
+                failure_run_result_obj = {"expectation_suite_severity_level": "failure"}
                 failure_validation_result = batch.validate(
                     failure_expectation_suite,
                     result_format="SUMMARY",
                     evaluation_parameters=evaluation_parameters,
                 )
-                return_obj["failure"][failure_validation_result_id][
-                    "validation_result"
-                ] = failure_validation_result
+                failure_run_result_obj["validation_result"] = failure_validation_result
                 failure_actions_results = self._run_actions(
                     batch,
                     failure_expectation_suite_identifier,
@@ -509,9 +607,8 @@ class WarningAndFailureExpectationSuitesValidationOperator(
                     failure_validation_result,
                     run_id,
                 )
-                return_obj["failure"][failure_validation_result_id][
-                    "actions_results"
-                ] = failure_actions_results
+                failure_run_result_obj["actions_results"] = failure_actions_results
+                run_results[failure_validation_result_id] = failure_run_result_obj
 
                 if not failure_validation_result.success and self.stop_on_first_error:
                     break
@@ -540,15 +637,13 @@ class WarningAndFailureExpectationSuitesValidationOperator(
                 )
 
             if warning_expectation_suite:
-                return_obj["warning"][warning_validation_result_id] = {}
+                warning_run_result_obj = {"expectation_suite_severity_level": "warning"}
                 warning_validation_result = batch.validate(
                     warning_expectation_suite,
                     result_format="SUMMARY",
                     evaluation_parameters=evaluation_parameters,
                 )
-                return_obj["warning"][warning_validation_result_id][
-                    "validation_result"
-                ] = warning_validation_result
+                warning_run_result_obj["validation_result"] = warning_validation_result
                 warning_actions_results = self._run_actions(
                     batch,
                     warning_expectation_suite_identifier,
@@ -556,26 +651,35 @@ class WarningAndFailureExpectationSuitesValidationOperator(
                     warning_validation_result,
                     run_id,
                 )
-                return_obj["warning"][warning_validation_result_id][
-                    "actions_results"
-                ] = warning_actions_results
+                warning_run_result_obj["actions_results"] = warning_actions_results
+                run_results[warning_validation_result_id] = warning_run_result_obj
 
-        return_obj["success"] = all(
-            [val["validation_result"].success for val in return_obj["failure"].values()]
+        validation_operator_result = ValidationOperatorResult(
+            run_id=run_id,
+            run_results=run_results,
+            validation_operator_config=self.validation_operator_config,
+            evaluation_parameters=evaluation_parameters,
+            success=all(
+                [
+                    run_result_obj["validation_result"].success
+                    for run_result_obj in run_results.values()
+                ]
+            ),
         )
 
-        # NOTE: Eugene: 2019-09-24: Update the data doc sites?
         if self.slack_webhook:
             if (
                 self.notify_on == "all"
                 or self.notify_on == "success"
-                and return_obj.success
+                and validation_operator_result.success
                 or self.notify_on == "failure"
-                and not return_obj.success
+                and not validation_operator_result.success
             ):
-                slack_query = self._build_slack_query(run_return_obj=return_obj)
+                slack_query = self._build_slack_query(
+                    validation_operator_result=validation_operator_result
+                )
                 send_slack_notification(
                     query=slack_query, slack_webhook=self.slack_webhook
                 )
 
-        return return_obj
+        return validation_operator_result
